
from orangewidget.widget import OWBaseWidget, Output
from orangewidget.settings import Setting
from orangewidget import gui
from orangewidget.utils.widgetpreview import WidgetPreview
import Orange.data
from Orange.data.pandas_compat import table_from_frame
import pandas as pd
import flat_table
import requests
from datetime import datetime

# Mobile stations

def get_mobile_stations():
    url = "http://api.canair.io:8080/tracks"
    stations = requests.get(url, verify=False).json()
    df_stations = pd.DataFrame(stations).transpose()
    df_stations = flat_table.normalize(df_stations)
    df_stations['index'] = pd.Categorical(df_stations['index'])
    df_stations['name'] = pd.Categorical(df_stations['name'])
    df_stations['deviceId'] = pd.Categorical(df_stations['deviceId'])
    df_stations.rename(columns=lambda s: s.replace("lastSensorData.", ""), inplace=True)
    df_stations.drop(columns=['dsl', 'lbl', 'pdist', 'PAX', 'main'], inplace=True)
    df_stations.lastLat = df_stations.lastLat.astype(float)
    df_stations.lastLon = df_stations.lastLon.astype(float)
    df_stations['size'] = df_stations["size"].astype(int)
    df_stations['distance'] = df_stations["distance"].astype(float)
    df_stations['tmp'] = df_stations["tmp"].astype(float)
    df_stations['pre'] = df_stations["pre"].astype(float)
    df_stations[[
        'hum', 
        'P4', 
        'P1', 
        'CO2T', 
        'CO2H', 
        'CO2', 
        'spd', 
        'lon', 
        'lat', 
        'alt', 
        'P25', 
        'P10'
        ]] = df_stations[[
            'hum', 
            'P4', 
            'P1', 
            'CO2T', 
            'CO2H', 
            'CO2', 
            'spd', 
            'lon', 
            'lat', 
            'alt', 
            'P25', 
            'P10'
            ]].astype(float)
    return df_stations

class CanairioWidget(OWBaseWidget):
    
    # Widget's name as displayed in the canvas
    name = "CanAIRio Mobile"

    # Short widget description
    description = "Get observations from mobile stations through CanAIRio API"

    # An icon resource file path for this widget
    icon = "icons/canairio_logo_rosa.png"

    # Priority in the section MECODA
    priority = 8

    # Basic (convenience) GUI definition:
    #   a simple 'single column' GUI layout
    want_main_area = False
    
    #   with a fixed non resizable geometry.
    resizing_enabled = False

    # Defining settings

    # Widget's outputs; here, a single output named "Observations", of type Table
    class Outputs:
        observations = Output("Observations", Orange.data.Table, auto_summary=False)

    def __init__(self):
        # use the init method from the class OWBaseWidget
        super().__init__()
        
        # info area
        info = gui.widgetBox(self.controlArea, "Info")

        self.infoa = gui.widgetLabel(info, 'No observations fetched yet.')
        self.infob = gui.widgetLabel(info, '')

        gui.separator(self.controlArea)

        # commit area
        self.commitBox = gui.widgetBox(self.controlArea, "", spacing=2)
        gui.button(self.commitBox, self, "Commit", callback=self.commit)

    def info_searching(self):
        self.infoa.setText('Searching...')
    
    # function to change subtype items due to type choice

    def commit(self):
        self.infoa.setText(f'Searching...')
        self.infob.setText(f'')
        try:            

            # show progress bar
            progress = gui.ProgressBar(self, 2)
            progress.advance()
            
            observations = get_mobile_stations()

            if len(observations) > 0:
                
                table_canairio = table_from_frame(observations)

                self.infoa.setText(f'{len(observations)} observations gathered')
                self.info.set_output_summary(len(observations))

                self.Outputs.observations.send(table_canairio)

            else:
                self.infoa.setText(f'Nothing found.')
                self.info.set_output_summary(self.info.NoOutput)

        except ValueError:
            self.infoa.setText(f'Nothing found.')
            
        except Exception as error:
            self.infoa.setText(f'ERROR: \n{error}')
            
        progress.finish()    

# For developer purpose, allow running the widget directly with python
if __name__ == "__main__":
    WidgetPreview(CanairioWidget).run()



import pandas as pd
import requests
import pandas as pd
import numpy as np
import nltk
nltk.download('vader_lexicon')
from nltk.sentiment.vader import SentimentIntensityAnalyzer

def get_odor_list():
    base_api = "https://odourcollect.eu/api"
    url = f"{base_api}/odor/list"
    odour_list = requests.get(url).json()['content']
    df = pd.DataFrame(odour_list)
    return df

def get_comments_origins(df):
    comments = {}
    origins = {}

    for id_ in df['id']:
        print(id_)
        try:
            comment = requests.get(f"https://odourcollect.eu/api/odor/{id_}").json()['object']['description']
            origin = requests.get(f"https://odourcollect.eu/api/odor/{id_}").json()['object']['origin']

            if origin is not None:
                origins[id_] = origin
            else:
                origins[id_] = np.nan
            
            if comment is not None:
                comments[id_] = comment
            else:
                comments[id_] = np.nan

        except:
            origins[id_] = np.nan
            comments[id_] = np.nan

    df['comments'] = df['id'].apply(lambda x: comments[x])
    df['origin'] = df['id'].apply(lambda x: origins[x])
    
    return df

df = get_odor_list()
df_full = get_comments_origins(df)

df_comments = df_full[df_full['comments'].notnull()]
df_comments = df_comments[['id', 'comments']]

sid = SentimentIntensityAnalyzer()

df_comments[['neg', 'neu', 'pos', 'compound']] = df_comments['comments'].apply(sid.polarity_scores).apply(pd.Series)

df_comments.to_csv("odour_collect_20220428_sentiment_analysis.csv", index=False)




from orangewidget.widget import OWBaseWidget, Output
from orangewidget.settings import Setting
from orangewidget import gui
from orangewidget.utils.widgetpreview import WidgetPreview
import Orange.data
from Orange.data.pandas_compat import table_from_frame
import pandas as pd
from mecoda_minka import get_obs, get_dfs, get_taxon_columns
import requests

#taxon_tree = pd.read_csv("taxon_tree.csv")
taxon_url = "https://raw.githubusercontent.com/eosc-cos4cloud/mecoda-orange/master/mecoda_orange/data/taxon_tree_with_marines.csv"
taxon_tree = pd.read_csv(taxon_url)

def get_descendants(selected_taxon, df):
    id_ = df[df['taxon_name'] == selected_taxon]['taxon_id'].item()
    ancestry = df[df['taxon_name'] == selected_taxon]['ancestry'].item()
    result_df = df[df['ancestry'] == f"{ancestry}/{id_}"][['taxon_id', 'taxon_name', 'rank']]
    names = result_df.taxon_name.to_list()
    taxa = ["", ]
    for name in names:
        id_taxa = result_df[result_df['taxon_name'] == name]['taxon_id'].item()
        taxa_url = f"https://minka-sdg.org/taxa/{id_taxa}.json"
        obs_count = requests.get(taxa_url).json()['observations_count']
        if obs_count > 0:
            order = result_df[result_df['taxon_name'] == name]['rank'].item().capitalize()
            taxa.append(f"{order} {name}")
    return sorted(taxa)

class TaxonWidget(OWBaseWidget):
    
    # Widget's name as displayed in the canvas
    name = "Minka Taxon Filter"

    # Short widget description
    description = "Get observations from Minka filtered by taxonomic level."

    # An icon resource file path for this widget
    icon = "icons/arrow-down-wide-short-solid-minka.png"

    # Priority in the section MECODA
    priority = 3

    # Basic (convenience) GUI definition:
    #   a simple 'single column' GUI layout
    want_main_area = False
    
    #   with a fixed non resizable geometry.
    resizing_enabled = False

    # Defining settings
    kingdom = Setting("", schema_only=True)
    filo = Setting("", schema_only=True)
    class_ = Setting("", schema_only=True)
    order = Setting("", schema_only=True)
    family = Setting("", schema_only=True)
    gender = Setting("", schema_only=True)
    species = Setting("", schema_only=True)
    selected = Setting("", schema_only=True)

    # Widget's outputs; here, a single output named "Observations", of type Table
    class Outputs:
        observations = Output("Observations", Orange.data.Table, auto_summary=False)
        photos = Output("Photos", Orange.data.Table, auto_summary=False)

    def __init__(self):
        # use the init method from the class OWBaseWidget
        super().__init__()
        
        # info area
        info = gui.widgetBox(self.controlArea, "Info")

        self.infoa = gui.widgetLabel(info, '\
            Here you can select taxonomic levels<br>\
            to filter observations from <a href="https://minka-sdg.org">minka-sdg.org</a>.')
        self.infob = gui.widgetLabel(info, 'No observations fetched yet.')

        gui.separator(self.controlArea)

        # searchBox area
        self.searchBox = gui.widgetBox(
            self.controlArea, 
            "Taxa fields",
            )

        self.searchBox.setFixedSize(300,230)

        self.kingdom_line = gui.comboBox(
            self.searchBox, 
            self, 
            "kingdom", 
            box=None,
            #label="First Level:", 
            labelWidth=None,
            items=(
                '',
                'Kingdom Animalia',
                'Kingdom Plantae',
                'Kingdom Fungi',
                'Kingdom Protozoa',
                'Kingdom Chromista',
                'Kingdom Bacteria',
                #'Kingdom Archae',
                ),
            sendSelectedValue=True,
            emptyString=False,
            editable=False,
            contentsLength=None, 
            searchable=True, 
            orientation=2,
            callback=self.kingdom_edit 
            )

        self.filo_line = gui.comboBox(
            self.searchBox, 
            self, 
            "filo", 
            box=None,
            #label="", 
            labelWidth=None,
            items=(),
            orientation=1, 
            sendSelectedValue=True,
            searchable=True,
            callback=self.filo_edit
            )
        
        self.class_line = gui.comboBox(
            self.searchBox, 
            self, 
            "class_", 
            box=None,
            #label="Class:", 
            labelWidth=None,
            items=(),
            orientation=1, 
            sendSelectedValue=True,
            searchable=True,
            callback=self.class_edit
            )

        self.order_line = gui.comboBox(
            self.searchBox, 
            self, 
            "order", 
            box=None,
            labelWidth=None,
            items=(),
            orientation=1, 
            sendSelectedValue=True,
            searchable=True,
            callback=self.order_edit
            )

        self.family_line = gui.comboBox(
            self.searchBox, 
            self, 
            "family", 
            box=None,
            labelWidth=None,
            items=(),
            orientation=1, 
            sendSelectedValue=True,
            searchable=True,
            callback=self.family_edit
            )

        self.gender_line = gui.comboBox(
            self.searchBox, 
            self, 
            "gender", 
            box=None,
            labelWidth=None,
            items=(),
            orientation=1, 
            sendSelectedValue=True,
            searchable=True,
            callback=self.gender_edit
            )

        self.species_line = gui.comboBox(
            self.searchBox, 
            self, 
            "species", 
            box=None,
            labelWidth=None,
            items=(),
            orientation=1, 
            sendSelectedValue=True,
            searchable=True,
            )

        # commit area
        self.commitBox = gui.widgetBox(self.controlArea, "", spacing=2)
        gui.button(self.commitBox, self, "Commit", callback=self.commit)

    def info_searching(self):
        self.infoa.setText('Searching...')
    
    # function to change subtype items due to type choice
    def kingdom_edit(self):
        self.infoa.setText(f'Selected: {self.kingdom}')
        self.infob.setText('')

        if self.kingdom != "":
            kingdom = self.kingdom.split(" ")[1]
            self.filo_line.clear()
            self.filo_line.addItems(get_descendants(kingdom, taxon_tree))
            self.selected = kingdom
        
    
    def filo_edit(self):
        self.infoa.setText(f'Selected: {self.kingdom} \n>> {self.filo}')
        self.infob.setText('')
        
        if self.filo != "":
            filo = self.filo.split(" ")[1]
            self.class_line.clear()
            self.class_line.addItems(get_descendants(filo, taxon_tree))
            self.selected = filo
        

    def class_edit(self):
        self.infoa.setText(f'Selected: {self.kingdom} \n>> {self.filo} \n>> {self.class_}')
        self.infob.setText('')
        
        if self.class_ != "":
            class_ = self.class_.split(" ")[1]
            self.order_line.clear()
            self.order_line.addItems(get_descendants(class_, taxon_tree))
            self.selected = class_

    def order_edit(self):
        self.infoa.setText(f'Selected: {self.kingdom} \n>> {self.filo} \n>> {self.class_} \n>> {self.order}')
        self.infob.setText('')
        if self.order != "":
            order = self.order.split(" ")[1]
            self.family_line.clear()
            self.family_line.addItems(get_descendants(order, taxon_tree))
            self.selected = order

    def family_edit(self):
        self.infoa.setText(f'Selected: {self.kingdom} \n>> {self.filo} \n>> {self.class_} \n>> {self.order} \n>> {self.family}')
        self.infob.setText('')
        if self.family != "":
            family = self.family.split(" ")[1]
            self.gender_line.clear()
            self.gender_line.addItems(get_descendants(family, taxon_tree))
            self.selected = family

    def gender_edit(self):
        self.infoa.setText(f'Selected: {self.kingdom} \n>> {self.filo} \n>> {self.class_} \n>> {self.order} \n>> {self.family} \n>> {self.gender}')
        self.infob.setText('')
        if self.gender != "":
            gender = self.gender.split(" ")[1]
            self.species_line.clear()
            self.species_line.addItems(get_descendants(gender, taxon_tree))
            self.selected = gender

    def species_edit(self):
        self.infoa.setText(f'Selected: {self.kingdom} \n>> {self.filo} \n>> {self.class_} \n>> {self.order} \n>> {self.family} \n>> {self.gender} \n>> {self.species}')
        self.infob.setText('')
        if self.species != "":
            species = self.species.replace("Species ", "")
            self.selected = species

    def commit(self):
        self.infoa.setText(f'Searching...')
        self.infob.setText(f'')
        try:
            if self.species != "":
                species = self.species.replace("Species ", "")
                self.selected = species
            # show progress bar
            progress = gui.ProgressBar(self, 2)
            progress.advance()
            id_selected = taxon_tree[taxon_tree['taxon_name'] == self.selected].taxon_id.item()

            taxa_url = f"https://minka-sdg.org/taxa/{id_selected}.json"
            obs_count = requests.get(taxa_url).json()['observations_count']

            if obs_count > 0:
                obs = get_obs(taxon_id=id_selected)             

                if len(obs) > 0:
                    self.df_obs, self.df_photos = get_dfs(obs)
                    
                    self.df_obs = get_taxon_columns(self.df_obs)
                    self.df_obs.taxon_name = pd.Categorical(self.df_obs.taxon_name)
                    self.df_obs.order = pd.Categorical(self.df_obs.order)
                    self.df_obs.family = pd.Categorical(self.df_obs.family)
                    self.df_obs.genus = pd.Categorical(self.df_obs.genus)
                    
                    self.df_obs['taxon_name'] = self.df_obs['taxon_name'].str.lower()
                    self.df_photos['taxon_name'] = self.df_photos['taxon_name'].str.lower()

                    self.infoa.setText(f'{len(obs)} observations gathered')
                    self.infob.setText(f'{len(self.df_photos)} photos gathered')
                    
                    self.Outputs.observations.send(table_from_frame(self.df_obs))

                    table_photos = table_from_frame(self.df_photos)
                    for meta in table_photos.domain.metas:
                        if meta.name == "photos.medium_url":
                            meta.attributes = {"type": "image"}

                    self.Outputs.photos.send(table_photos)
                    self.info.set_output_summary(len(obs))

                else:
                    self.infoa.setText(f'Nothing found.')
                    self.infob.setText("")
                    self.info.set_output_summary(self.info.NoOutput)


        except ValueError:
            self.infoa.setText(f'Nothing found.')
            
        except Exception as error:
            self.infoa.setText(f'ERROR: \n{error}')
            
        progress.finish()    

# For developer purpose, allow running the widget directly with python
if __name__ == "__main__":
    WidgetPreview(TaxonWidget).run()



from orangewidget.widget import OWBaseWidget, Output
from orangewidget.settings import Setting
from orangewidget import gui
from orangewidget.utils.widgetpreview import WidgetPreview
import Orange.data
from Orange.data.pandas_compat import table_from_frame
import pandas as pd
import flat_table
import requests

# Fixed stations

def get_fixed_stations_data(filter=None) -> pd.DataFrame:
    url = "http://api.canair.io:8080/dwc/stations"
    stations = requests.get(url).json()
    df_stations = pd.DataFrame(stations)
    df_stations = flat_table.normalize(df_stations)
    df_stations.rename(columns=lambda s: s.replace("measurements.", ""), inplace=True)
    df_stations['observedOn'] = pd.to_datetime(df_stations['observedOn'])
    df_stations = df_stations.drop(['index'], axis=1)
    df_stations.station_name = pd.Categorical(df_stations.station_name)
    if filter is not None:
        if filter != "":
            df_stations = df_stations[df_stations['measurementType'] == filter]
            df_stations.measurementValue = df_stations.measurementValue.astype(float)
    return df_stations


class CanairioWidget(OWBaseWidget):
    
    # Widget's name as displayed in the canvas
    name = "CanAIRio Fixed"

    # Short widget description
    description = "Get observations from fixed stations through CanAIRio API"

    # An icon resource file path for this widget
    icon = "icons/canairio_logo_gris.png"

    # Priority in the section MECODA
    priority = 6

    # Basic (convenience) GUI definition:
    #   a simple 'single column' GUI layout
    want_main_area = False
    
    #   with a fixed non resizable geometry.
    resizing_enabled = True

    # Defining settings
    type = Setting('', schema_only=True)

    # Widget's outputs; here, a single output named "Observations", of type Table
    class Outputs:
        observations = Output("Observations", Orange.data.Table, auto_summary=False)

    def __init__(self):
        # use the init method from the class OWBaseWidget
        super().__init__()
        
        # info area
        info = gui.widgetBox(self.controlArea, "Info")

        self.infoa = gui.widgetLabel(info, 'No observations fetched yet.')
        self.infob = gui.widgetLabel(info, '')

        gui.separator(self.controlArea)

        # searchBox area
        self.searchBox = gui.widgetBox(self.controlArea, "Search fields")

        self.type_line = gui.comboBox(
            self.searchBox, 
            self, 
            "type", 
            box=None,
            label="Measurement type:", 
            labelWidth=None,
            items=(
                '',
                'PM1', 
                'PM2.5', 
                'PM10', 
                'Temperature', 
                'Humidity', 
                'Pressure',
                'CO2', 
                'CO2 Temperature', 
                'CO2 Humidity'
                ),
            sendSelectedValue=True,
            emptyString=False,
            editable=False,
            contentsLength=None, 
            searchable=True, 
            orientation=1,
            )

        # commit area
        self.commitBox = gui.widgetBox(self.controlArea, "", spacing=2)
        gui.button(self.commitBox, self, "Commit", callback=self.commit)

    def info_searching(self):
        self.infoa.setText('Searching...')
    
    # function to change subtype items due to type choice

    def commit(self):
        self.infoa.setText(f'Searching...')
        self.infob.setText(f'')
        try:            
            if self.type == "":
                self.type is None

            # show progress bar
            progress = gui.ProgressBar(self, 2)
            progress.advance()
            
            observations = get_fixed_stations_data(self.type)

            # convert lon-lat variables to float, convert time column to hour, min, sec columns
            #observations[['decimalLatitude', 'decimalLongitude']] = observations[['decimalLatitude', 'decimalLongitude']].astype(float)

            if len(observations) > 0:
                
                table_canairio = table_from_frame(observations)

                self.infoa.setText(f'{len(observations)} observations gathered')
                self.info.set_output_summary(len(observations))

                self.Outputs.observations.send(table_canairio)

            else:
                self.infoa.setText(f'Nothing found.')
                self.info.set_output_summary(self.info.NoOutput)

        except ValueError:
            self.infoa.setText(f'Nothing found.')
            
        except Exception as error:
            self.infoa.setText(f'ERROR: \n{error}')
            
        progress.finish()    

# For developer purpose, allow running the widget directly with python
if __name__ == "__main__":
    WidgetPreview(CanairioWidget).run()



import pandas as pd
import os
import datetime
from ictiopy import ictiopy

from orangewidget.widget import OWBaseWidget, Output
from orangewidget.settings import Setting
from orangewidget import gui, widget
from orangewidget.utils.widgetpreview import WidgetPreview
import Orange.data
from Orange.data.pandas_compat import table_from_frame

from PyQt5.QtWidgets import QFileDialog
from AnyQt.QtWidgets import QStyle, QSizePolicy as Policy

def clean_df(observations):
    observations.weight = observations.weight.astype(float)
    observations.price_local_currency = observations.price_local_currency.astype(float)
    observations.num_photos = observations.num_photos.astype(int)
    observations.fishing_duration = observations.fishing_duration.astype(float)
    observations.num_of_fishers = observations.num_of_fishers.astype(float)
    observations.number_of_fish = observations.number_of_fish.astype(float)
    observations.obs_year = observations.obs_year.astype(int)
    observations.obs_month = observations.obs_month.astype(int)
    observations.obs_day = observations.obs_day.astype(int)
    return observations

def split_date(observations, init, end):
    observations['obs_date'] = observations['obs_year'].astype(int).astype(str) + observations['obs_month'].astype(int).astype(str).str.zfill(2) + observations['obs_day'].astype(int).astype(str).str.zfill(2)
    observations['obs_date'] = pd.to_datetime(observations['obs_date'])
    observations = observations[observations['obs_date'] >= init]
    observations = observations[observations['obs_date'] <= end]
    observations = observations.drop(['obs_date'], axis=1)
    return observations

class IctioWidget(OWBaseWidget):
    
    # Widget's name as displayed in the canvas
    name = "Ictio Observations"

    # Short widget description
    description = "Get observations from Ictio, a mobile phone app created to register observations of caught fish in the Amazon basin"

    # An icon resource file path for this widget
    icon = "icons/ictio-circular.png"

    # Priority in the section MECODA
    priority = 10

    # Basic (convenience) GUI definition:
    #   a simple 'single column' GUI layout
    want_main_area = False
    
    #   with a fixed non resizable geometry.
    resizing_enabled = False

    # Defining settings
    path_folder = Setting("", schema_only=True)
    path_file = Setting("", schema_only=True)
    date_init = Setting("1860-01-01", schema_only=True)
    date_end = Setting(str(datetime.date.today()), schema_only=True)
    #observations = Setting("", schema_only=True)

    # Widget's outputs; here, a single output named "Observations", of type Table
    class Outputs:
        observations = Output("Observations", Orange.data.Table, auto_summary=False)


    def __init__(self):
        # use the init method from the class OWBaseWidget
        super().__init__()

        # info area
        info = gui.widgetBox(self.controlArea, "Info")

        self.infoa = gui.widgetLabel(
            info, 
            'Please specify the path to a <b>Ictio_Basic_xxxxxxxx.zip</b> file\
            <br>that has been downloaded from the "Download" section<br>\
            of <a href="https://ictio.org/">ictio.org</a> website.<br><br>\
            You can also specity an XLSX file if you have already extracted\
            <br> the file BDB_XXXXXXXX.xlsx that can be found inside\
             <br>ICTIO_BASIC_XXXXXXXX.zip'
)
        self.infob = gui.widgetLabel(info, 'NOTE: Downloading the file requires user registration.')

        gui.separator(self.controlArea)

        # searchBox area
        self.searchBox = gui.widgetBox(self.controlArea, "Source")
        self.browsers = gui.widgetBox(self.searchBox, "", orientation=1)
        zip_button = gui.button(
            self.browsers, 
            self, 
            'Choose .zip', 
            callback=self.browse_zip, 
            autoDefault=False,
            width=160,
            )
        zip_button.setIcon(self.style().standardIcon(QStyle.SP_DirOpenIcon))
        zip_button.setSizePolicy(
            Policy.Maximum, 
            Policy.Fixed
            )
        file_button = gui.button(
            self.browsers, 
            self, 
            'Choose .xlsx', 
            callback=self.browse_file, 
            autoDefault=False,
            width=160,
            )
        file_button.setIcon(self.style().standardIcon(QStyle.SP_DirOpenIcon))
        file_button.setSizePolicy(
            Policy.Maximum, 
            Policy.Fixed
            )

        #layout.addWidget(file_button, 0, 2)

        """self.path_line = gui.lineEdit(
            self.searchBox, 
            self, 
            "path", 
            label="Path to zip:", 
            orientation=1, 
            controlWidth=300,
            disable=True
            )"""

        #gui.separator(self.searchBox)
        self.dateBox = gui.widgetBox(self.controlArea, "Filter by date")

        self.date_init_line = gui.lineEdit(
            self.dateBox, 
            self, 
            "date_init", 
            label="Initial Date:", 
            orientation=1, 
            controlWidth=140,
            )

        self.date_end_line = gui.lineEdit(
            self.dateBox, 
            self, 
            "date_end", 
            label="End Date:", 
            orientation=1, 
            controlWidth=140
            )

        # commit area
        self.commitBox = gui.widgetBox(self.controlArea, "", spacing=2)
        gui.button(self.commitBox, self, "Load", callback=self.commit)

    def info_searching(self):
        self.infoa.setText('Loading...')

    def browse_file(self):
        dialog = QFileDialog()
        home = os.path.expanduser("~")
        path_string, __ = dialog.getOpenFileName(self, 'Select a zip file', home, "xlsx files (*.xlsx)")
        self.path_file = path_string

        if self.path_file is not None:
            try:
                #file_selected = path_string.split("data")[-1]
                self.infoa.setText(f"<b>File selected:</b><br>{path_string}")
                self.infob.setText("")

            except ValueError:
                self.infoa.setText(f'Nothing found.')
                
            except Exception as error:
                self.infoa.setText(f'ERROR: \n{error}')
                self.infob.setText("")
                print(error)

        else:
            self.infoa.setText(f'Choose some xlsx file to load data.')

    def browse_zip(self):
        dialog = QFileDialog()
        home = os.path.expanduser("~")
        path_string, __ = dialog.getOpenFileName(self, 'Select a zip file', home, "Zip files (*.zip)")
        self.path_folder = path_string

        if self.path_folder is not None:
            try:
                #file_selected = path_string.split("data")[-1]
                self.infoa.setText(f"<b>File selected:</b><br>{path_string}")
                self.infob.setText("")

            except ValueError:
                self.infoa.setText(f'Nothing found.')
                
            except Exception as error:
                self.infoa.setText(f'ERROR: \n{error}')
                self.infob.setText("")
                print(error)

        else:
            self.infoa.setText(f'Choose some zip file to load data.')


    def commit(self):
        self.infoa.setText(f'Loading...')
        self.infob.setText(f'(This could take a while, be patient)')
        try:
            # convert date_init and date_end to datetime format
            if type(self.date_init) == str:
                init = datetime.datetime.strptime(self.date_init, '%Y-%m-%d')
            else:
                init = self.date_init

            if type(self.date_end) == str:
                end = datetime.datetime.strptime(self.date_end, '%Y-%m-%d')
            else:
                end = self.date_end

            # show progress bar
            progress = gui.ProgressBar(self, 2)
            progress.advance()
            
            if self.path_file != "":
                directory, file = os.path.split(os.path.abspath(self.path_file))
                observations = ictiopy.sanitizedb(
                    ictiopy.load_ictio_bdb_file(
                        directory, 
                        file
                        )
                    )
            elif self.path_folder != "":
                observations = ictiopy.load_zipdb(self.path_folder) 
            
            observations = clean_df(observations)
            observations = split_date(observations, init, end)

            if len(observations) > 0:
                
                table_ictio = table_from_frame(observations)

                self.infoa.setText(f'{len(observations):,} observations gathered')
                self.infob.setText("")

                self.info.set_output_summary(len(observations))

                self.Outputs.observations.send(table_ictio)

            else:
                self.infoa.setText(f'Nothing found.')
                self.info.set_output_summary(self.info.NoOutput)

        except ValueError:
            self.infoa.setText(f'Nothing found.')
            
        except Exception as error:
            self.infoa.setText(f'ERROR: \n{error}')
            self.infob.setText("")
            print(error)
            
        progress.finish()    
        

# For developer purpose, allow running the widget directly with python
if __name__ == "__main__":
    WidgetPreview(IctioWidget).run()


from orangewidget.widget import OWBaseWidget, Output, Input
from orangewidget.settings import Setting
from orangewidget import gui
from orangewidget.utils.widgetpreview import WidgetPreview
import Orange.data
from Orange.data.pandas_compat import table_from_frame, table_to_frame
from mecoda_minka import get_obs, get_dfs
import requests 
import numpy as np


class ExtraInfoWidget(OWBaseWidget):
    # Widget's name as displayed in the canvas
    name = "Minka Extra Info"
    # Short widget description
    description = "Get extra information from Minka observations"

    # An icon resource file path for this widget
    # (a path relative to the module where this widget is defined)
    icon = "icons/circle-info-solid-minka.png"
    priority = 4

    # Basic (convenience) GUI definition:
    #   a simple 'single column' GUI layout
    want_main_area = False
    #   with a fixed non resizable geometry.
    resizing_enabled = False
    
    # We want the current number entered by the user to be saved and restored when saving/loading a workflow. 
    # We can achieve this by declaring a special property/member in the widget’s class definition like so: 
    id_obs = Setting(0)
    #project_name = Setting(None)

    # Widget's outputs; here, a single output named "Number", of type int

    class Inputs:
        data = Input("Data", Orange.data.Table, auto_summary=False)

    class Outputs:
        extra_data = Output("extra", Orange.data.Table, auto_summary=False)


    want_main_area = False

    def __init__(self):
        super().__init__()

        # GUI
        box = gui.widgetBox(self.controlArea, "Info")
        self.infoa = gui.widgetLabel(box, 'No data on input yet, waiting to get something.')
        self.infob = gui.widgetLabel(box, '')

    @Inputs.data
    def set_data(self, dataset):
        if dataset is not None:
            self.infoa.setText(f'{len(dataset)} instances in input dataset')
            df = table_to_frame(dataset)
            ids = df['id'].to_list()
            dic = {}

            progress = gui.ProgressBar(self, len(df))
            
            for id_num in ids:
                url = f'https://minka-sdg.org/observations/{id_num}.json'
                page = requests.get(url, verify=False)

                idents = page.json()['identifications']
                
                if len(idents) > 0:
                    user_identification = idents[0]['user']['login']
                    first_taxon_name = idents[0]['taxon']['name']
                    last_taxon_name = idents[len(idents) - 1]['taxon']['name']
                    dic[id_num] = [user_identification, first_taxon_name, last_taxon_name]
                    progress.advance()
                else:
                    dic[id_num] = [0, 0, 0]
                    progress.advance()
            progress.finish()    

            df['first_identification'] = df['id'].apply(lambda x: str(dic[x][0]))
            df['first_taxon_name'] = df['id'].apply(lambda x: str(dic[x][1]))
            df['last_taxon_name'] = df['id'].apply(lambda x: str(dic[x][2]))

            df['first_taxon_match'] = np.where(df['first_taxon_name'] == df['last_taxon_name'], 'True', 'False')
            df['first_identification_match'] = np.where(df['first_identification'] == df['user_login'], 'True', 'False')
            
            out = table_from_frame(df)
            
            self.infob.setText(f'Observations gathered: {len(df)}')

            self.Outputs.extra_data.send(out)

        else:
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')
            #self.Outputs.sample.send("Sampled Data")

if __name__ == "__main__":
    WidgetPreview(ExtraInfoWidget).run()




from setuptools import setup, find_packages
import os


# incluir documentación de https://github.com/biolab/orange3-example-addon/blob/master/setup.py

NAME = "Orange3-MECODA"
DOCUMENTATION_NAME = 'Orange MECODA'

VERSION = "1.4.2"

AUTHOR = "Ana Alvarez, ICM-CSIC"
AUTHOR_EMAIL = "ana.alvarez@icm.csic.es"

URL = 'https://github.com/eosc-cos4cloud/mecoda-orange'

DESCRIPTION = "Add-on containing MECODA widgets to analyse data from citizen science observatories"
LONG_DESCRIPTION  = open(os.path.join(os.path.dirname(__file__), 'README_pypi.md')).read()
LICENSE = "BSD"

KEYWORDS = [
    'orange3 add-on',
    'orange',
    'data mining',
    ]
#PACKAGES = find_packages(include=("mecoda_orange*",))
setup(
    name="Mecoda Orange",
    version=VERSION,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    license=LICENSE,
    packages=["mecoda_orange"],
    #packages=PACKAGES,
    package_data={
        "mecoda_orange": ["icons/*"], 
        },
    classifiers=[
        "License :: OSI Approved :: European Union Public Licence 1.2 (EUPL 1.2)",
        'Environment :: X11 Applications :: Qt',
        'Environment :: Console',
        'Environment :: Plugins',
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Development Status :: 4 - Beta",
        'Topic :: Scientific/Engineering :: Visualization',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Operating System :: OS Independent',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        ],
    entry_points={
        "orange.widgets": "MECODA = mecoda_orange",
        "orange.addons": "MECODA = mecoda_orange"
        },
    install_requires=[
        "flat_table >= 1.1.1",
        "mecoda-nat >= 0.5.8",
        "pandas >= 1.4.1",
        "Orange3 >= 3.31.1",
        "pyodourcollect >= 1.0.0",
        "ictiopy >= 1.0.0",
        "mecoda-minka >= 0.3.2",
        "mooda >= 1.13.1"
        ],
    keywords=KEYWORDS,
    include_package_data=True,
    zip_safe=False,
    )


